#include "utils.h"

static gboolean
gunichar_ok (gunichar c)
{
  return
    (c != (gunichar) -2) &&
    (c != (gunichar) -1);
}

#define octet_escape 0x009a

jsval safe_make_js_string(JSContext *cx, const char* st) {
  JSString *jst = NULL;
  if (!st || g_utf8_validate(st, -1, NULL)) {
    jst = JS_NewStringCopyZ(cx, st);
  }
  else {
    GString *string;
    unsigned char c;
    const gchar *end;

    end = st + strlen (st);
    string = g_string_new(NULL);

    while ((c = *st) != 0) {
      if (c >= 0x80 &&
          gunichar_ok(g_utf8_get_char_validated(st, end - st))) {
          int len = g_utf8_skip [c];
          g_string_append_len(string, st, len);
          st += len;
      }
      else if (c < 0x80) {
	g_string_append_c(string, c);
	st++;
      }
      else {
	g_string_append_unichar(string, octet_escape);
	g_string_append_unichar(string, c);
	st++;
      }
    }

    jst = JS_NewStringCopyZ(cx, string->str);
    g_string_free(string, TRUE);
  }
  if (jst)
    return STRING_TO_JSVAL(jst);
  else
    return JSVAL_VOID;
}

gchar*
decode_js_string(JSContext* cx, JSString* jst) {
  char *bs, *str, *ret;
  GString *string;
  unsigned char c;
  const gchar *end;
  if (!jst) return NULL;
  JS_BeginRequest(cx);
  str = bs = JS_EncodeString(cx, jst);
  if (!strchr(str, octet_escape)) {
    ret = g_strdup(str);
    goto out;
  }
  
  string = g_string_new(NULL);
  end = str + strlen (str);

  while ((c = *str) != 0) {
    if (c >= 0x80) {
      gunichar uc = g_utf8_get_char_validated(str, end - str);
      int len = g_utf8_skip [c];
      if (uc == octet_escape) {
	str += len;
	uc = g_utf8_get_char_validated(str, end - str);
	g_string_append_c(string, uc);
	str += g_utf8_skip [(unsigned char)*str];
      }
      else {
	g_string_append_len(string, str, len);
	str += len;
      }
    }
    else {
      g_string_append_c(string, c);
      str++;
    }
  }

  ret = string->str;
  g_string_free(string, FALSE);
 out:
  JS_free(cx, bs);
  JS_EndRequest(cx);
  return ret;
}
